#include <stdio.h>
#include <stdlib.h>

#define INF (-1)
#define NILL (-1)
#define TRUE 1
#define FALSE 0

double potroseno = 0;

int jeManji(int i, int j) {
    if(i == INF) return FALSE;
    if(j == INF) return TRUE;

    return i < j;
}

int nadjiMinimum(double *d, int *obidjen, int brGradova) {
    int i, min, minCvor;
    min = INF;
    minCvor = NILL;

    for(i = 0; i < brGradova; i++) {
        if(obidjen[i] == FALSE && jeManji(d[i], min)) {
            min = d[i];
            minCvor = i;
        }
    }

    return minCvor;
}

void relax(double **graf, double *d, int *pret, int u, int v) {
    if(jeManji(graf[u][v]+d[u], d[v])) {
        d[v] = graf[u][v]+d[u];
        pret[v] = u;
    }
}

void dijkstra(double **graf, double *d, int *pret, int *obidjen, int brGradova) {
    int i, u, v;

    for(i = 0; i < brGradova; i++) {
        u = nadjiMinimum(d, obidjen, brGradova);
        obidjen[u] = TRUE;

        for(v = 0; v < brGradova; v++) {
            if(graf[u][v] != INF && obidjen[v] == FALSE) {
                relax(graf, d, pret, u, v);
            }
        }
    }
}

int nazadNadjiMinimum(double *d, int *obidjen, int *prodjeni, int brGradova) {
    int i, min, minCvor;
    min = INF;
    minCvor = NILL;

    for(i = 0; i < brGradova; i++) {
        if(prodjeni[i] == FALSE && obidjen[i] == FALSE && jeManji(d[i], min)) {
            min = d[i];
            minCvor = i;
        }
    }

    return minCvor;
}

void nazadDijkstra(double **graf, double *d, int *pret, int *obidjen, int *prodjeni, int brGradova) {
    int i, u, v;

    for(i = 0; i < brGradova; i++) {
        u = nazadNadjiMinimum(d, obidjen, prodjeni, brGradova);
        obidjen[u] = TRUE;

        for(v = 0; v < brGradova; v++) {
            if(graf[u][v] != INF && obidjen[v] == FALSE && prodjeni[i] == FALSE) {
                relax(graf, d, pret, u, v);
            }
        }
    }
}

void rekonstrukcija(int g, int s, double **graf, int *prodjeni, int *pret) {
    if(g == s) return;
    else {
        prodjeni[g] = TRUE;
        potroseno += graf[g, pret[g]];
        rekonstrukcija(pret[g], s, graf, prodjeni, pret);
        printf("%d ", g);
        return;
    }
}

int main() {
    double **graf;
    double *d;
    int *obidjen, *pret;
    int i, j;

    double cenaGoriva, potrosnjaAutoput, potrosnjaRegionalni;
    int brGradova, brVeza;

    scanf("%lf", &cenaGoriva);
    scanf("%lf%lf", &potrosnjaAutoput, &potrosnjaRegionalni);

    scanf("%d%d", &brGradova, &brVeza);

    graf = (double**)malloc(brGradova*sizeof(double*));
    d = (double*)malloc(brGradova*sizeof(double*));
    obidjen = (int*)malloc(brGradova*sizeof(int*));
    pret = (int*)malloc(brGradova*sizeof(int*));

    for(i = 0; i < brGradova; i++) {
        graf[i] = (double*)malloc(brGradova*sizeof(double));
        d[i] = INF;
        pret[i] = NILL;
        obidjen[i] = FALSE;
    }

    for(i = 0; i < brGradova; i++) {
        for(j = 0; j < brGradova; j++) {
            graf[i][j] = INF;
        }
    }

    // Cena veza
    int u, v, tip, putarina;
    double l;
    for(i = 0; i < brVeza; i++) {
        scanf("%d%d%lf%d", &u, &v, &l, &tip);
        double cena;
        if(tip == 0) cena = cenaGoriva * (double)(potrosnjaRegionalni) / 100.00 * l;
        else {
            scanf("%d", &putarina);
            cena = cenaGoriva * (double)(potrosnjaAutoput) / 100.00 * l + putarina;
        }
        graf[u][v] = cena;
        graf[v][u] = cena;
    }

    int s, g;

    scanf("%d%d", &s, &g);

    d[u] = 0;

    dijkstra(graf, d, pret, obidjen, brGradova);

    int *prodjeni;
    prodjeni = (int*)malloc(brGradova*sizeof(int));

    for(i = 0; i < brGradova; i++) {
        prodjeni[i] = FALSE;
    }

    rekonstrukcija(g, s, graf, prodjeni, pret);

    double *d2;
    int *pret2, *obidjen2;

    d2 = (double*)malloc(brGradova*sizeof(double));
    pret2 = (int*)malloc(brGradova*sizeof(int));
    obidjen2 = (int*)malloc(brGradova*sizeof(int));

    for(i = 0; i < brGradova; i++) {
        d2[i] = INF;
        pret2[i] = INF;
        obidjen2[i] = FALSE;
    }

    nazadDijkstra(graf, d2, pret2, obidjen2, prodjeni, brGradova);

    rekonstrukcija(s, g, graf, prodjeni, pret2);

    return 0;
}