#include<stdio.h>
#include<stdlib.h>

#define BESKONACNO -1
#define NEDEFINISANO -1

#define TRUE 1
#define FALSE 0

#define REGIONALNIPUT 0
#define AUTOPUT 1

typedef struct veza
{
	double cena;
	double duzina;
	int tip;
}Veza;

int jeManji(int i, int j)
{
	if(i == BESKONACNO)
		return FALSE;
	
	if(j == BESKONACNO)
		return TRUE;
		
	return i < j;
}


int nadjiMinimum(int *obidjen, double *d, int n)
{
	int i;
	int min = BESKONACNO;
	int minIndeks = NEDEFINISANO;
	
	for(i=0; i<n; i++)
		if(obidjen[i] == FALSE && jeManji(d[i], min))
		{
			min = d[i];
			minIndeks = i;
		}
	
	return minIndeks;
}

void relax(Veza **graf, double *d, int *pret, int u, int v)
{
	if(jeManji(d[u]+graf[u][v].cena, d[v]))
	{
		d[v] = d[u]+graf[u][v].cena;
		pret[v] = u;
	}
}

void dijkstra(Veza **graf, int *obidjen, double *d, int *pret, int n)
{
	int i;
	int u, v;
	
	for(i=0; i<n; i++)
	{
		u = nadjiMinimum(obidjen, d, n);
		obidjen[u] = TRUE;
		
		for(v=0; v<n; v++)
			if(obidjen[v] == FALSE && graf[u][v].cena != BESKONACNO) 
				relax(graf, d, pret, u, v);
	}
}


void rekonstrukcija(int i, int polaziste, int *pret, int *obidjen, double *cenaPutovanja, double *kilometraza, Veza **graf)
{
	if(i == polaziste)
	{
		//printf("%lf\n", *cenaPutovanja);
		//printf("%lf\n", *kilometraza);
		printf("%d ", polaziste);
	}
	else
	{
		(*cenaPutovanja) += graf[pret[i]][i].cena;
		(*kilometraza) += graf[pret[i]][i].duzina;
		obidjen[i] = FALSE;
		rekonstrukcija(pret[i], polaziste, pret, obidjen, cenaPutovanja, kilometraza, graf);
		printf("%d ", i);
	}
}


void rekonstrukcija2(int i, int polaziste, int *pret, int *obidjen, double *cenaPutovanja, double *kilometraza, Veza **graf)
{
	if(i != polaziste && obidjen[i] == TRUE)
	{
		//printf("%lf\n", *cenaPutovanja);
		//printf("%lf\n", *kilometraza);
		//printf("polaziste: %d\n", polaziste);

		(*cenaPutovanja) += graf[pret[i]][i].cena;
		(*kilometraza) += graf[pret[i]][i].duzina;
		rekonstrukcija2(pret[i], polaziste, pret, obidjen, cenaPutovanja, kilometraza, graf);
		printf("%d ", i);
	}
	else
	{
		printf("%d ", polaziste);
	}
}

int main()
{
	int i, j;
	
	double cenaGoriva;
		scanf("%lf", &cenaGoriva);
		
	double potrosnjaAutoput, potrosnjaRegPut;
		scanf("%lf%lf", &potrosnjaAutoput, &potrosnjaRegPut);
	
	int N; // gradovi
	int M; // putevi
	
	scanf("%d%d", &N, &M);
	
	// ALOKACIJA
	Veza **graf = (Veza**)malloc(N*sizeof(Veza*));
	double *dO = (double*)malloc(N*sizeof(double));
	int *pretO = (int*)malloc(N*sizeof(int));
	int *obidjenO = (int*)malloc(N*sizeof(int));
	double *dP = (double*)malloc(N*sizeof(double));
	int *pretP = (int*)malloc(N*sizeof(int));
	int *obidjenP = (int*)malloc(N*sizeof(int));
	
	for(i=0; i<N; i++)
	{
		graf[i] = (Veza*)malloc(N*sizeof(Veza));
		
		// INICIJALIZACIJA
		for(j=0; j<N; j++)
			graf[i][j].cena = BESKONACNO;
			
		dO[i] = BESKONACNO;
		pretO[i] = NEDEFINISANO;
		obidjenO[i] = FALSE;
		
		dP[i] = BESKONACNO;
		pretP[i] = NEDEFINISANO;
		obidjenP[i] = FALSE;
	}

	
	int u, v, oznaka;
	double l; // duzina puta
	double putarina;
	double cA, cR;
	for(i=0; i<M; i++)
	{
		scanf("%d%d%lf%d", &u, &v, &l, &oznaka);
		
		if(oznaka == AUTOPUT)
		{
			scanf("%lf", &putarina);
			cA = cenaGoriva/potrosnjaAutoput*l + putarina;
		}
		else
			cR = cenaGoriva/potrosnjaRegPut*l;
		
		if(cA < cR)
		{
			graf[u][v].cena = graf[v][u].cena = cA;
			graf[u][v].duzina = graf[v][u].duzina = l;
		}
		else
		{
			graf[u][v].cena = graf[v][u].cena = cR;
			graf[u][v].duzina = graf[v][u].duzina = l;
		}
		
	}
	
	int polaziste, odrediste;
		scanf("%d%d", &polaziste, &odrediste);
	

	dO[polaziste] = 0;
	dijkstra(graf, obidjenO, dO, pretO, N);
	
	
	printf("d: \n");
	for(i=0; i<N; i++)
		printf("%lf\n", dO[i]);
		
	printf("pret: \n");
	for(i=0; i<N; i++)
		printf("%d\n", pretO[i]);
		
		/*
	printf("obidjen: \n");
	for(i=0; i<N; i++)
		printf("%d\n", obidjen[i]);*/
	
	/*	
	printf("\n");
	for(i=0; i<N; i++)
	{
		for(j=0; j<N; j++)
			printf("%.2lf\t", graf[i][j].cena);
		
		printf("\n");
	}*/
	
	double cenaPutovanja = 0.0;
	double kilometraza = 0.0;
	printf("\n");
	rekonstrukcija(odrediste, polaziste, pretO, obidjenO, &cenaPutovanja, &kilometraza, graf);
	
	dP[odrediste] = 0;
	dijkstra(graf, obidjenP, dP, pretP, N);
	rekonstrukcija2(polaziste, odrediste, pretP, obidjenO, &cenaPutovanja, &kilometraza, graf);
	
	printf("\n%lf\n", cenaPutovanja);
	printf("\n%lf\n", kilometraza);
	printf("\n");
	
	return 0;
}
