#include <stdio.h>
#include <stdlib.h>

typedef struct
{
	int autoput;
	double Duzina;
	double putarina;
}Put;

#define NILL -2
#define INF -1
#define true 1
#define false 0

int*AlocirajNizINT(int n)
{
	return (int*)malloc(n * sizeof(int));
}

double *AlocirajNizDOUBLE(int n)
{
	return (double*)malloc(n * sizeof(double));
}

int DaLiJeManji(int a, int b)
{
	if(a == INF)
		return false;
	else if(b == INF)
		return true;
	else
		return a < b;
}

int NadjiNajmanji(int *obidjeni, double *cenePuteva, int n)
{
	int minPozicija;
	double min = INF;

	for(int i = 0; i < n; i++)
	{
		if(obidjeni[i] == false && DaLiJeManji(cenePuteva[i], min))
		{
			min = cenePuteva[i];
			minPozicija = i;
		}
	}

	return minPozicija;
}

void Dikstra(double **graf, int *obidjeni, int *prethodni, double *cenePuteva, int n)
{
	int u;

	for(int i = 0; i < n; i++)
	{
		u = NadjiNajmanji(obidjeni, cenePuteva, n);
		obidjeni[u] = true;

		for(int v = 0; v < n; v++)
		{
			if(obidjeni[v] == false && graf[u][v] != INF && DaLiJeManji(cenePuteva[u] + graf[u][v], cenePuteva[v]))
			{
				cenePuteva[v] = cenePuteva[u] + graf[u][v];
				prethodni[v] = u;
			}
		}
	}
}

double **KreirajGraf(Put **putevi,int n, double potrosnjAutoput, double potrosnjaRegionalniput)
{
	double **graf = (double**) malloc(n * sizeof(double*));

	for(int i = 0; i < n; i++)
    {
        graf[i] = AlocirajNizDOUBLE(n);

		for(int j = 0; j < n; j++)
		{
			if(putevi[i][j].Duzina == INF)
                graf[i][j] = INF;
            else
            {
                if(putevi[i][j].autoput == 1)
                {
                    graf[i][j] = putevi[i][j].Duzina * (potrosnjAutoput/100) + putevi[i][j].putarina;
                }
                else
                {
                    graf[i][j] = putevi[i][j].Duzina * (potrosnjaRegionalniput/100);
                }
            }
		}
	}

	return graf;
}

int Rekonstrukcija(int *prethodni, int pocetak, int kraj, int* cvorovi)
{
    int p = 0;

    while(kraj != pocetak)
    {
        cvorovi[p] = kraj;
        p++;
        kraj = prethodni[kraj];
    }
    cvorovi[p] = pocetak;
    p++;

    return p;
}

int main()
{
    double cenaGiruva;
    double potrosnjAutoput, potrosnjaRegionalniput;

    scanf("%lf", &cenaGiruva);
    scanf("%lf%lf", &potrosnjAutoput, &potrosnjaRegionalniput);

    int n,m;
    scanf("%d %d", &n, &m);

	Put **putevi = (Put**) malloc(n * sizeof(Put*));

    for(int i = 0; i < n; i++)
    {
		putevi[i] = (Put*) malloc(n * sizeof(Put));
		for(int j = 0; j < n; j++)
		{
		    putevi[i][j].autoput = 0;
            putevi[i][j].Duzina = INF;
            putevi[i][j].putarina = INF;
		}
	}


    int *obidjeni = AlocirajNizINT(n);
    int *prethodni = AlocirajNizINT(n);
    double *cenaPuta = AlocirajNizDOUBLE(n);

    for(int i = 0; i < n; i++)
    {
		obidjeni[i] = false;
		prethodni[i] = NILL;
		cenaPuta[i] = INF;
	}

    for(int i = 0; i < m; i++)
    {
		int u,v,a;
		double l, p;
		scanf("%d%d%lf%d",&u,&v,&l,&a);

        p = INF;

		if(a == 1)
		{
			scanf("%lf", &p);
		}

        putevi[u][v].autoput = a;
        putevi[u][v].Duzina = l;
        putevi[u][v].putarina = p;

        putevi[v][u].autoput = a;
        putevi[v][u].Duzina = l;
        putevi[v][u].putarina = p;
    }

    double **graf = KreirajGraf(putevi, n, potrosnjAutoput, potrosnjaRegionalniput);

	int polaziste, odrediste;
	scanf("%d%d", &polaziste, &odrediste);

    cenaPuta[polaziste] = 0;
    Dikstra(graf, obidjeni, prethodni, cenaPuta, n);


    int *cvoroviDoGrada = AlocirajNizINT(n + m);

    int p1 = Rekonstrukcija(prethodni, polaziste, odrediste, cvoroviDoGrada);

    double cenaPutovanja = 0;
    double kilometrazaPutovanja = 0;

    if( prethodni[odrediste] != NILL && cenaPuta[odrediste] != INF)
    {
        cenaPutovanja = cenaPuta[odrediste];
    }

    for(int i = p1 - 1; i > 0; i--)
    {
        kilometrazaPutovanja += putevi[cvoroviDoGrada[i]][cvoroviDoGrada[i - 1]].Duzina;
        for(int j = 0; j < n; j++)
        {
            putevi[cvoroviDoGrada[i]][j].putarina = INF;
            putevi[cvoroviDoGrada[i]][j].Duzina = INF;
            putevi[cvoroviDoGrada[i]][j].autoput = 0;

            putevi[j][cvoroviDoGrada[i]].putarina = INF;
            putevi[cvoroviDoGrada[j]][cvoroviDoGrada[i]].Duzina = INF;
            putevi[cvoroviDoGrada[j]][cvoroviDoGrada[i]].autoput = 0;
        }
    }

//Vraca se nazad
    graf = KreirajGraf(putevi, n, potrosnjAutoput, potrosnjaRegionalniput);
    for(int i = 0; i < n; i++)
    {
		obidjeni[i] = false;
		prethodni[i] = NILL;
		cenaPuta[i] = INF;
	}

    cenaPuta[odrediste] = 0;
    Dikstra(graf, obidjeni, prethodni, cenaPuta, n);

    int *cvoroviOdGrada = AlocirajNizINT(n + m);

    int p2 = Rekonstrukcija(prethodni, odrediste, polaziste, cvoroviOdGrada);

    if( prethodni[polaziste] != NILL && cenaPuta[polaziste] != INF)
    {
        cenaPutovanja += cenaPuta[polaziste];
    }

    for(int i = p2 - 1; i > 0; i--)
    {
        kilometrazaPutovanja += putevi[cvoroviOdGrada[i]][cvoroviOdGrada[i - 1]].Duzina;
    }

    if(p1 != 0 && p2 != 0)
    {
        printf("%lf\n",cenaPutovanja);
        printf("%lf\n",kilometrazaPutovanja);

        for(int i = p1 - 1; i >= 0; i--)
            printf("%d ", cvoroviDoGrada[i]);

        for(int i = p2 - 1; i >= 0; i--)
            printf("%d ", cvoroviOdGrada[i]);

        printf("\n");
    }
    else
        printf("%d\n",-1);


    return 0;
}
